#ifndef FileSelectionFrame_h
#define FileSelectionFrame_h

// $Id: FileSelectionFrame.h,v 1.60 2014/04/11 08:48:07 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "CustomComboBox.h"
#include "FileInfoModel.h"
#include "FileSelectionClipboard.h"
#include "FtpMacro.h"
#include "XmlOptions.h"

#include <QAbstractItemView>
#include <QItemSelectionModel>
#include <QMenu>

// forward declarations
class PathEditor;
class BaseFileInfoToolTipWidget;

//! document/folder navigation
class FileSelectionFrame: public QWidget, public Counter
{

    //! Qt meta object declaration
    Q_OBJECT

    public:

    //! constructor
    FileSelectionFrame( QWidget*, BaseFileInfo::TypeFlags );

    //!@name accessors
    //@{

    //! current path
    FileInfo path( void ) const;

    //! type
    BaseFileInfo::TypeFlags type( void ) const
    { return type_; }

    //! selected files
    FileInfo::List selection( void ) const;

    //! returns true if this widget owns argument
    bool owns( QWidget* ) const;

    //! active state
    bool isActive( void ) const
    { return active_; }

    //! list
    QAbstractItemView& list( void ) const
    { return *list_; }


    //! view mode
    enum ViewMode
    {
        List,
        Icons
    };

    //! mode
    ViewMode viewMode( void ) const
    { return viewMode_; }

    //! model
    const FileInfoModel& model( void ) const
    { return model_; }

    //! path editor
    PathEditor& pathEditor( void ) const
    { return *pathEditor_; }

    //@}

    //!@name modifiers
    //@{

    //! home path
    void setHomePath( const File& ) const;

    //! root path list
    void setRootPathList( const File::List& ) const;

    //! current path
    void setPath( const File& );

    // clear
    void clear( void );

    //! flags
    enum Flag
    {
        None = 0,
        Visible = 1<<1,
        Selected = 1<<2
    };

    Q_DECLARE_FLAGS( Flags, Flag );

    //! add File in list
    void add( const FileInfo&, Flags = None );

    //! add Files in list
    void add( const FileInfo::List& files, Flags flags = None )
    { foreach( const FileInfo& file, files ) add( file, flags ); }

    //! remove File from list
    void remove( const FileInfo& );

    //! remove Files from list
    void remove( const FileInfo::List& files )
    { foreach( const FileInfo& file, files ) remove( file ); }

    //! active state
    bool setActive( bool value )
    {
        if( active_ == value ) return false;
        active_ = value;
        _updateBackgroundColors();
        return true;
    }

    //! mode
    bool setViewMode( ViewMode );

    //! model
    FileInfoModel& model( void )
    { return model_; }

    //@}

    //!@name actions
    //@{

    //! toggle visibility action
    QAction& visibilityAction( void ) const
    { return *visibilityAction_; }

    //! previous directory
    QAction& reloadAction( void ) const
    { return *reloadAction_; }

    //! previous directory
    QAction& previousDirectoryAction( void ) const
    { return *previousDirectoryAction_; }

    //! next directory
    QAction& nextDirectoryAction( void ) const
    { return *nextDirectoryAction_; }

    //! directory above
    QAction& parentDirectoryAction( void ) const
    { return *parentDirectoryAction_; }

    //! home directory
    QAction& homeDirectoryAction( void ) const
    { return *homeDirectoryAction_; }

    // new folder action
    QAction& newFolderAction( void ) const
    { return *newFolderAction_; }

    //! cut selection action
    QAction& cutAction( void ) const
    { return *cutAction_; }

    // copy selection action
    QAction& copyAction( void ) const
    { return *copyAction_; }

    // paste clipboard action
    QAction& pasteAction( void ) const
    { return *pasteAction_; }

    // paste clipboard action
    QAction& pasteInFolderAction( void ) const
    { return *pasteInFolderAction_; }

    // rename selection action
    QAction& renameAction( void ) const
    { return *renameAction_; }

    // remove selection action
    QAction& removeAction( void ) const
    { return *removeAction_; }

    //! edit action
    QAction& openAction( void ) const
    { return *openAction_; }

    // file properties
    QAction& filePropertiesAction( void ) const
    { return *filePropertiesAction_; }

    //@}

    Q_SIGNALS:

    //! request for opening a file
    void fileSelected( FileInfo );

    //! request for opening multiple files
    void openFilesRequested( const FileInfo::List& );

    //! clipboard changed
    void clipboardChanged( const FileSelectionClipboard& );

    //! emitted when actions enable state has changed
    void actionsChanged( void );

    //! request command
    void macroRequested( const FtpMacro& );

    //! request commands
    void macrosRequested( const FtpMacro::List& );

    //! request paste from clipboard
    void pasteClipboardRequested( const FileInfo& );

    //! request file information
    void requestFileInformation( const FileInfo&, const QIcon& );

    public Q_SLOTS:

    //! hidden files
    void setShowHiddenFiles( bool value )
    {
        if( showHiddenFiles_ == value ) return;
        showHiddenFiles_ = value;
        _reload();
    }

    //! navigator
    void setShowNavigator( bool value )
    {
        const FileInfo navigator( File(".."), BaseFileInfo::TypeFlags( FileInfo::Folder|FileInfo::Navigator|type_ ) );
        if( value ) model_.add( navigator );
        else model_.remove( navigator );
    }

    //! clear selection
    void clearSelection( void );

    //! select item
    void select( const FileInfo& );

    protected Q_SLOTS:

    //! index activated
    void _indexActivated( const QModelIndex& );

    //! reload current directory
    void _reload( void );

    //! new folder
    void _newFolder( void );

    //! select home directory
    void _homeDirectory( void );

    //! cut
    void _cut( void );

    //! copy
    void _copy( void );

    //! paste
    void _paste( void );

    //! paste
    void _pasteInFolder( void );

    //! rename
    void _rename( void );

    //! remove
    void _remove( void );

    //! open
    void _open( void );

    //! file properties
    void _fileProperties( void );

    //! update list menu
    void _updateContextMenu( const QPoint& );

    //! update actions, based on selection
    void _updateActions( void );

    //! navigation actions
    void _updateNavigationActions( void );

    //! show tooltip
    void _showToolTip( const QModelIndex& );

    //! toggle visibility
    void _toggleVisibility( bool value )
    {
        setVisible( value );
        XmlOptions::get().set<bool>( visibilityOptionName_, value );
    }

    protected:

    //! true if local
    bool _isLocal( void ) const
    { return !( type_ & FileInfo::Remote ); }

    //! true if remote
    bool _isRemote( void ) const
    { return type_ & FileInfo::Remote; }

    //! setup list
    void _setupList( void );

    //! show hidden files
    const bool& _showHiddenFiles( void ) const
    { return showHiddenFiles_; }

    //! update background colors
    void _updateBackgroundColors( void ) const;

    private:

    //! actions
    void _installActions( void );

    //! path line editor
    PathEditor* pathEditor_;

    //! tooltip manager
    BaseFileInfoToolTipWidget* toolTipWidget_;

    //! model
    FileInfoModel model_;

    //! selection model
    QItemSelectionModel selectionModel_;

    //! list of files
    QAbstractItemView* list_;

    //!@name menus
    //@{

    //! column selection
    QMenu* columnSelectionMenu_;

    //! column sorting
    QMenu* columnSortingMenu_;

    //@}

    //!@name actions
    //@{

    //! toggle visibility
    QAction* visibilityAction_;

    //! reload
    QAction* reloadAction_;

    //! previous directory action
    QAction* previousDirectoryAction_;

    //! next directory action
    QAction* nextDirectoryAction_;

    //! parent directory action
    QAction* parentDirectoryAction_;

    //! home directory action
    QAction* homeDirectoryAction_;

    //! new folder action
    QAction* newFolderAction_;

    //! cut selection action
    QAction* cutAction_;

    //! copy selection action
    QAction* copyAction_;

    //! paste
    QAction* pasteAction_;

    //! paste in folder action
    QAction* pasteInFolderAction_;

    //! rename action
    QAction* renameAction_;

    //! remove selection
    QAction* removeAction_;

    //! edit action
    QAction* openAction_;

    //! file properties
    QAction* filePropertiesAction_;

    //@}

    //! base type (bitwise or of FileInfo::Type
    /*! this is used to distinguish between local and remote list views */
    BaseFileInfo::TypeFlags type_;

    //! visibility option name
    QString visibilityOptionName_;

    //! view mode option name
    QString viewModeOptionName_;

    //! mode
    ViewMode viewMode_;

    //! show/hide hidden files
    bool showHiddenFiles_;

    //! active state
    bool active_;

};

Q_DECLARE_OPERATORS_FOR_FLAGS( FileSelectionFrame::Flags )

#endif
